/*
	PING Server for XFBB by Stewart Wilkinson G0LGS 
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h> 
#include <time.h>
#include <sys/stat.h>

#include "utils.h"

FILE *init_ptr, *msg_ptr, *out_ptr, *hlp_ptr;
char buffer[255];
char from_call[10];
char from_bbs[80];
char subject[80];
char mail_in[128];
char sys_name[20];
char sys_call[10];
char bbs_call[80];
char conf_path[128];
char config[128];
char Text[80];
int  nparams;

time_t time_now;
struct tm *now;

#ifdef V700

#define TAIL "\n*** PING V" Version " " Vdate " for XFBB 7.00 by G0LGS ***\n"
#define INIT "init.srv"

int ReadInit()
{
	#define BBS_CALL 1
	#define SYSOP_NAME 11
	#define SYSOP_CALL 12
	#define MAIL_IN 14

	int end, index;

	/* Open init.srv */
 	init_ptr = fopen( INIT, "rt");
	if (init_ptr == NULL) {
		fprintf( stderr, "Unable to open '%s'\n", INIT );
		return 1; 
	}

	/* Read the lines we need . */
	index=end=0;
	while (!end && fgets(buffer, sizeof(buffer), init_ptr) ) {

		/* Comments ignored */
		if (*buffer == '#')
			continue;

		switch (++index) {

			case BBS_CALL :
				sscanf( buffer, "%[^\n]\n", bbs_call);
				*strstr(bbs_call,".") = '\0';
				break;

			case SYSOP_NAME :
				sscanf( buffer, "%s", sys_name);
				break;

			case SYSOP_CALL :
				sscanf( buffer, "%s", sys_call);
				break;

			case MAIL_IN :
				sscanf( buffer, "%s", mail_in);
				end = 1;
				break;
		}
	}

	fclose(init_ptr);

	return 0;
}
#else

#define TAIL "\n*** PING V" Version " " Vdate " for XFBB 7.01+ by G0LGS ***\n"
#define FBBCONF "/etc/ax25/fbb.conf"

int ReadInit()
{
	int len, i;
	int end = 0;
	char buffer[256];
	char *scan;
	char *ptr;
	char key[20];
	char pathname[256];

	if( (ptr = getenv("FBBCONF")) == NULL ){
		strcpy( pathname, FBBCONF);
	}else{
		strcpy( pathname, ptr);
	}

	if ((init_ptr = fopen (pathname, "r")) != NULL) {

		while( !end && fgets(buffer, sizeof(buffer), init_ptr) ){

			scan = buffer;

			/* Eat Spaces */
			while (isspace(*scan))
				++scan;

			/* Comments & Blanks ignored */
			if (*scan == '#' || *scan == '\0' )
				continue;

			/* remove leading \n */
			len = strlen(scan);
			if( len && (scan[len-1] == '\n'))
				scan[len-1] = '\0';

			/* keyword */
			i=0;
			ptr = key;
			while( isgraph(*scan)) {
				if( i++ < 4 ) {
					*ptr++ = *scan;
				}
				++scan;
				if(*scan == '=')
					break;         
			}
			*ptr = '\0';

			/* find the '=' */
			scan = strchr(scan, '=');

			/* copy value */
			if(scan) {
				++scan;
				while( isspace(*scan))
					++scan;

				/* if 1st char is '"' then string */
				if( *scan == '"' ){
					++scan;
					ptr = strrchr(scan,'"');
					if(ptr)
						*ptr = '\0';
				}

				if( strcmp( key, "call" ) == 0 ) {
					strcpy( bbs_call, scan );
					*strstr( bbs_call, "." ) = '\0';

				}else if( strcmp( key, "name" ) == 0 ) {
					strcpy( sys_name, scan );

				}else if( strcmp( key, "syso" ) == 0 ) {
					strcpy( sys_call, scan );

				}else if( strcmp( key, "impo" ) == 0 ) {
					strcpy( mail_in, scan );

				}else if( strcmp( key, "conf" ) == 0 ) {
					strcpy( conf_path, scan );
				}

			}
		}

		fclose(init_ptr);
  
	}else{
		fprintf( stderr, "Unable to open '%s'\n", pathname );
		return 1;
	}

	return 0;
}

#endif

int main(int argc, char **argv)
{
	int end = 0;
	int msg_pos = 0;
	int Date, Time;

	/* Check the Arguments */
	if (argc != 2){
		fprintf( stderr, TAIL"\n" );
		exit(1);
	}

	if( ReadInit() != 0 ){
		fprintf( stderr, "Error Processing Config file\n" );
		exit(1);
        }

	/* Open the Message */
	msg_ptr = fopen(argv[1], "r");
	if (msg_ptr == NULL){
		fprintf( stderr, "Cannot Open '%s'\n", argv[1] );
		exit(1);
	}

	/* Read the From line */
	fgets(buffer, sizeof(buffer), msg_ptr);
	sscanf(buffer, "%*s %*s %*s %s", from_call );

	/* Read the Subject */
	fgets(buffer, sizeof(buffer), msg_ptr);
	sscanf(buffer, "%[^\n]\n", subject);

	/* Remember where the message text starts */
	msg_pos = ftell( msg_ptr );

	/* Extract Original BBS from R: lines */
	end=0;
	while( ! end && (fgets( buffer, sizeof(buffer), msg_ptr ) ) ){

		/* R: lines */
		if( sscanf( buffer,"R:%d/%d%*[^@]@:%s", &Date, &Time, from_bbs ) >= 1 ) {
			msg_pos = ftell( msg_ptr );
			continue;

		/* Skip T: lines */
		}else if( sscanf( buffer,"T:%[^\n]\n", Text ) != 0 ) {
			msg_pos = ftell( msg_ptr );
			continue;

		/* End of R: / T: lines */
		}else{
			end =1;
		}
	}


	/* Get Current Time Information */
	time_now = time(NULL);
	now = localtime( &time_now );
	

	/* Open mail.in file */
	if( (out_ptr = fopen(mail_in, "a")) == NULL ) {
		fprintf( stderr, "Error appending to %s\n", mail_in );
		exit(1);
        }


	if( strlen(from_bbs) >= 6 ) {
		fprintf( out_ptr, "#\nSP %s @ %s < %s\n", from_call, from_bbs, bbs_call );
	}else{
		fprintf( out_ptr, "#\nSP %s < %s\n", from_call, bbs_call );
	}

	fprintf( out_ptr, "Re: %s\n", subject );

	/* Add PING Header */
	fprintf( out_ptr, "\nPing received by %s at %02d:%02d:%02d On %02d/%02d/%02d\n\n", bbs_call, now->tm_hour, now->tm_min, now->tm_sec, now->tm_mday, now->tm_mon+1, now->tm_year %100 );

	fseek( msg_ptr, msg_pos, SEEK_SET );

	while( fgets( buffer, sizeof(buffer), msg_ptr ) ){

		/* Ignore FBB From: line */
		if( strncmp( buffer, "From: ", 6 ) == 0 )
			continue;

		/* Ignore FBB To  : line */
		if( strncmp( buffer, "To  : ", 6 ) == 0 )
			continue;

		/* Ignore /ex */
		if( strncasecmp(buffer, "/ex", 3 ) == 0 )
			continue;

		fprintf( out_ptr, "> %s", buffer );

	}

	fclose(msg_ptr);

	fprintf( out_ptr, "%s", TAIL );
	fprintf( out_ptr, "/EX\n" );
	fclose( out_ptr );

	/* Delete the Input Message file (as XFBB doesn't) */
//	unlink(argv[1]);

	exit(0);
}

